/// <reference types="node" />
/// <reference types="node" />
import { Writable } from 'stream';
import { postgresqlErrorCodes } from './errors';
import { Query } from './query';
import { ConnectionOptions } from 'tls';
import { ResultIterator, ResultRecord } from './result';
import { ClientConnectionDefaults, ClientConnectionOptions, DatabaseError, ErrorLevel, TransactionStatus } from './protocol';
import { DataFormat, DataType, ValueTypeReader } from './types';
export type Connect = Error | null;
export type End = void;
export interface Parameter {
    name: string;
    value: string;
}
export interface ClientNotice extends DatabaseError {
    level: ErrorLevel;
    code: keyof typeof postgresqlErrorCodes;
    message: string;
}
export interface DataTypeError {
    dataType: DataType;
    value: any;
}
export declare enum SSLMode {
    Disable = "disable",
    Prefer = "prefer",
    Require = "require"
}
export interface SSL {
    mode: (SSLMode.Prefer | SSLMode.Require);
    options?: ConnectionOptions;
}
export interface Configuration extends Partial<ClientConnectionDefaults & ClientConnectionOptions> {
    user?: string;
    database?: string;
    host?: string;
    port?: number;
    password?: string;
    types?: Map<DataType, ValueTypeReader>;
    keepAlive?: boolean;
    preparedStatementPrefix?: string;
    connectionTimeout?: number;
    ssl?: (SSLMode.Disable | SSL);
}
export interface Notification {
    processId: number;
    channel: string;
    payload?: string;
}
export interface PreparedStatement<T = ResultRecord> {
    close: (portal?: string) => Promise<void>;
    execute: (values?: any[], portal?: string, format?: DataFormat | DataFormat[], streams?: Record<string, Writable>) => ResultIterator<T>;
}
export type Callback<T> = (data: T) => void;
/** A database client, opening a single connection to the database.
 *
 * @remarks
 * You must open the connection using {@link connect}, otherwise no query will be processed.
 */
export declare class Client {
    readonly config: Configuration;
    private readonly events;
    private ending;
    private connected;
    private connecting;
    private error;
    private readonly encoding;
    private readonly writer;
    private readonly clientNonce;
    private serverSignature;
    private expect;
    private stream;
    private mustDrain;
    private activeRow;
    private bindQueue;
    private closeHandlerQueue;
    private cleanupQueue;
    private errorHandlerQueue;
    private preFlightQueue;
    private rowDescriptionQueue;
    private parameterDescriptionQueue;
    private nextPreparedStatementId;
    private activeDataHandlerInfo;
    closed: boolean;
    processId: number | null;
    secretKey: number | null;
    transactionStatus: TransactionStatus | null;
    /**
    * @param config - An optional configuration object, comprised of connection details
    *     and client configuration. Most of the connection details can also be specified
    *     using environment variables, see {@link Environment}.
    */
    constructor(config?: Configuration);
    private startup;
    private listen;
    connect(): Promise<boolean>;
    end(): Promise<void>;
    on(event: 'connect', callback: Callback<Connect>): void;
    on(event: 'end', callback: Callback<End>): void;
    on(event: 'parameter', callback: Callback<Parameter>): void;
    on(event: 'notification', callback: Callback<Notification>): void;
    on(event: 'error', callback: Callback<DatabaseError>): void;
    on(event: 'notice', callback: Callback<ClientNotice>): void;
    prepare<T = ResultRecord>(text: string, name?: string, types?: DataType[]): Promise<PreparedStatement<T>>;
    query<T = ResultRecord>(text: string, values?: any[], types?: DataType[], format?: DataFormat | DataFormat[], streams?: Record<string, Writable>): ResultIterator<T>;
    private bindAndExecute;
    execute<T = ResultRecord>(query: Query): ResultIterator<T>;
    private handleError;
    private send;
    private sendUsing;
    private parseError;
    private receive;
}
